@@ -1,4 +1,5 @@
 /* This file is part of Spectacle, the KDE screenshot utility
+ * Copyright 2019 David Redondo <kde@david-redondo.de>
  * Copyright (C) 2015 Boudhayan Gupta <bgupta@kde.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,6 +22,9 @@
 
 #include "ExportManager.h"
 
+#include "settings.h"
+#include <kio_version.h>
+
 #include <QDir>
 #include <QMimeData>
 #include <QMimeDatabase>
@@ -34,14 +38,16 @@
 #include <QRegularExpression>
 #include <QRegularExpressionMatch>
 #include <QString>
+#include <QRandomGenerator>
 
 #include <KSharedConfig>
 #include <KIO/ListJob>
 #include <KIO/MkpathJob>
 #include <KIO/FileCopyJob>
 #include <KIO/StatJob>
+#include <KRecentDocument>
+
 
-#include "SpectacleConfig.h"
 
 ExportManager::ExportManager(QObject *parent) :
     QObject(parent),
@@ -49,9 +55,7 @@ ExportManager::ExportManager(QObject *parent) :
     mTempFile(QUrl()),
     mTempDir(nullptr)
 {
-    connect(this, &ExportManager::imageSaved, [](const QUrl &savedAt) {
-        SpectacleConfig::instance()->setLastSaveFile(savedAt);
-    });
+    connect(this, &ExportManager::imageSaved, &Settings::setLastSaveLocation);
 }
 
 ExportManager::~ExportManager()
@@ -87,7 +91,7 @@ Spectacle::CaptureMode ExportManager::captureMode() const
     return mCaptureMode;
 }
 
-void ExportManager::setCaptureMode(const Spectacle::CaptureMode &theCaptureMode)
+void ExportManager::setCaptureMode(Spectacle::CaptureMode theCaptureMode)
 {
     mCaptureMode = theCaptureMode;
 }
@@ -119,7 +123,8 @@ void ExportManager::setTimestamp(const QDateTime &timestamp)
 
 QString ExportManager::defaultSaveLocation() const
 {
-    QString savePath = SpectacleConfig::instance()->defaultSaveLocation().toLocalFile();
+    const QUrl saveUrl = Settings::self()->defaultSaveLocation();
+    QString savePath = saveUrl.scheme().isEmpty() ? saveUrl.toString() : saveUrl.toLocalFile();
     savePath = QDir::cleanPath(savePath);
 
     QDir savePathDir(savePath);
@@ -136,7 +141,7 @@ QUrl ExportManager::getAutosaveFilename()
     const QDir baseDirPath(baseDir);
     const QString filename = makeAutosaveFilename();
     const QString fullpath = autoIncrementFilename(baseDirPath.filePath(filename),
-                                                   SpectacleConfig::instance()->saveImageFormat(),
+                                                   Settings::self()->defaultSaveImageFormat().toLower(),
                                                    &ExportManager::isFileExists);
 
     const QUrl fileNameUrl = QUrl::fromUserInput(fullpath);
@@ -160,14 +165,14 @@ QString ExportManager::truncatedFilename(QString const &filename)
 
 QString ExportManager::makeAutosaveFilename()
 {
-    return formatFilename(SpectacleConfig::instance()->autoSaveFilenameFormat());
+    return formatFilename(Settings::self()->saveFilenameFormat());
 }
 
 QString ExportManager::formatFilename(const QString &nameTemplate)
 {
     const QDateTime timestamp = mPixmapTimestamp;
     QString baseName = nameTemplate;
-    const QString baseDir = defaultSaveLocation();
+    QString baseDir = defaultSaveLocation();
     QString title;
 
     if (mCaptureMode == Spectacle::CaptureMode::ActiveWindow ||
@@ -194,32 +199,40 @@ QString ExportManager::formatFilename(const QString &nameTemplate)
     // check if basename includes %[N]d token for sequential file numbering
     QRegularExpression paddingRE;
     paddingRE.setPattern(QStringLiteral("%(\\d*)d"));
-    QRegularExpressionMatch paddingMatch;
-
-    while (result.indexOf(paddingRE, 0, &paddingMatch) > -1) {
-        int highestFileNumber = 0;
-
-        // determine padding value
-        int paddedLength = 1;
-        if (!paddingMatch.captured(1).isEmpty()) {
-            paddedLength = paddingMatch.captured(1).toInt();
+    QRegularExpressionMatchIterator it = paddingRE.globalMatch(result);
+    if (it.hasNext()) {
+        // strip any subdirectories from the template to construct the filename matching regex
+        // we are matching filenames only, not paths
+        QString resultCopy = QRegularExpression::escape(result.section(QLatin1Char('/'), -1));
+        QVector<QRegularExpressionMatch> matches;
+        while (it.hasNext()) {
+            QRegularExpressionMatch paddingMatch = it.next();
+            matches.push_back(paddingMatch);
+            //determine padding value
+            int paddedLength = 1;
+            if (!paddingMatch.captured(1).isEmpty()) {
+                paddedLength = paddingMatch.captured(1).toInt();
+            }
+            QString escapedMatch = QRegularExpression::escape(paddingMatch.captured());
+            resultCopy.replace(escapedMatch, QStringLiteral("(\\d{%1,})").arg(QString::number(paddedLength)));
+        }
+        if (result.contains(QLatin1Char('/'))) {
+          // In case the filename template contains a subdirectory,
+          // we need to search for files in the subdirectory instead of the baseDir.
+          // so let's add that to baseDir before we search for files.
+          baseDir += QStringLiteral("/%1").arg(result.section(QLatin1Char('/'), 0, -2));
         }
-
         // search save directory for files
         QDir dir(baseDir);
         const QStringList fileNames = dir.entryList(QDir::Files, QDir::Name);
+        int highestFileNumber = 0;
 
         // if there are files in the directory...
-        if (fileNames.length() > 0) { 
-            QString resultCopy = result;
+        if (fileNames.length() > 0) {
             QRegularExpression fileNumberRE;
-            const QString replacement = QStringLiteral("(\\d{").append(QString::number(paddedLength)).append(QLatin1String(",})"));
-            const QString fullNameMatch = QStringLiteral("^").append(resultCopy.replace(paddingMatch.captured(),replacement)).append(QStringLiteral("\\..*$"));
-            fileNumberRE.setPattern(fullNameMatch);
-
+            fileNumberRE.setPattern(resultCopy);
             // ... check the file names for string matching token with padding specified in result
             const QStringList filteredFiles = fileNames.filter(fileNumberRE);
-
             // if there are files in the directory that look like the file name with sequential numbering
             if (filteredFiles.length() > 0) {
                 // loop through filtered file names looking for highest number
@@ -232,8 +245,14 @@ QString ExportManager::formatFilename(const QString &nameTemplate)
             }
         }
         // replace placeholder with next number padded
-        const QString nextFileNumberPadded = QString::number(highestFileNumber + 1).rightJustified(paddedLength, QLatin1Char('0'));
-        result.replace(paddingMatch.captured(), nextFileNumberPadded);
+        for (const auto& match : matches) {
+            int paddedLength = 1;
+            if (!match.captured(1).isEmpty()) {
+                paddedLength = match.captured(1).toInt();
+            }
+            const QString nextFileNumberPadded = QString::number(highestFileNumber + 1).rightJustified(paddedLength, QLatin1Char('0'));
+            result.replace(match.captured(), nextFileNumberPadded);
+        }
     }
 
     // Remove leading and trailing '/'
@@ -245,7 +264,7 @@ QString ExportManager::formatFilename(const QString &nameTemplate)
     }
 
     if (result.isEmpty()) {
-        result = SpectacleConfig::instance()->defaultFilename();
+        result = QStringLiteral("Screenshot");
     }
     return truncatedFilename(result);
 }
@@ -269,7 +288,7 @@ QString ExportManager::autoIncrementFilename(const QString &baseName, const QStr
     // unlikely this will ever happen, but just in case we've run
     // out of numbers
 
-    result = fileNameFmt.arg(QLatin1String("OVERFLOW-") + QString::number(qrand() % 10000));
+    result = fileNameFmt.arg(QLatin1String("OVERFLOW-") + QString::number(QRandomGenerator::global()->bounded(10000)));
     return truncatedFilename(result) + extension;
 }
 
@@ -279,7 +298,7 @@ QString ExportManager::makeSaveMimetype(const QUrl &url)
     QString type = mimedb.mimeTypeForUrl(url).preferredSuffix();
 
     if (type.isEmpty()) {
-        return SpectacleConfig::instance()->saveImageFormat();
+        return Settings::self()->defaultSaveImageFormat().toLower();
     }
     return type;
 }
@@ -287,7 +306,7 @@ QString ExportManager::makeSaveMimetype(const QUrl &url)
 bool ExportManager::writeImage(QIODevice *device, const QByteArray &format)
 {
     QImageWriter imageWriter(device, format);
-    imageWriter.setQuality(SpectacleConfig::instance()->compressionQuality());
+    imageWriter.setQuality(Settings::self()->compressionQuality());
     /** Set compression 50 if the format is png. Otherwise if no compression value is specified
      *  it will fallback to using quality (QTBUG-43618) and produce huge files.
      *  See also qpnghandler.cpp#n1075. The other formats that do compression seem to have it
@@ -389,7 +408,8 @@ QUrl ExportManager::tempSave()
         // supports the use-case of creating multiple screenshots in a row
         // and exporting them to the same destination e.g. via clipboard,
         // where the temp file name is used as filename suggestion
-        const QString baseFileName = mTempDir->path() + QDir::separator() + makeAutosaveFilename();
+        const QString baseFileName = mTempDir->path() + QLatin1Char('/') +
+                                     QUrl::fromLocalFile(makeAutosaveFilename()).fileName();
 
         QString mimetype = makeSaveMimetype(QUrl(baseFileName));
         const QString fileName = autoIncrementFilename(baseFileName, mimetype,
@@ -418,10 +438,16 @@ bool ExportManager::save(const QUrl &url)
     }
 
     QString mimetype = makeSaveMimetype(url);
+    bool saveSucceded = false;
     if (url.isLocalFile()) {
-        return localSave(url, mimetype);
+         saveSucceded = localSave(url, mimetype);
+    } else {
+        saveSucceded = remoteSave(url, mimetype);
     }
-    return remoteSave(url, mimetype);
+    if (saveSucceded) {
+        KRecentDocument::add(url, QGuiApplication::desktopFileName());
+    }
+    return saveSucceded;
 }
 
 bool ExportManager::isFileExists(const QUrl &url) const
@@ -429,8 +455,8 @@ bool ExportManager::isFileExists(const QUrl &url) const
     if (!(url.isValid())) {
         return false;
     }
+    KIO::StatJob * existsJob = KIO::statDetails(url, KIO::StatJob::DestinationSide, KIO::StatNoDetails, KIO::HideProgressInfo);
 
-    KIO::StatJob * existsJob = KIO::stat(url, KIO::StatJob::DestinationSide, 0);
     existsJob->exec();
 
     return (existsJob->error() == KJob::NoError);
@@ -454,7 +480,6 @@ void ExportManager::doSave(const QUrl &url, bool notify)
     if (save(savePath)) {
         QDir dir(savePath.path());
         dir.cdUp();
-        SpectacleConfig::instance()->setLastSaveFile(savePath);
 
         emit imageSaved(savePath);
         if (notify) {
@@ -466,7 +491,6 @@ void ExportManager::doSave(const QUrl &url, bool notify)
 bool ExportManager::doSaveAs(QWidget *parentWindow, bool notify)
 {
     QStringList supportedFilters;
-    SpectacleConfig *config = SpectacleConfig::instance();
 
     // construct the supported mimetype list
     const auto mimeTypes = QImageWriter::supportedMimeTypes();
@@ -475,12 +499,12 @@ bool ExportManager::doSaveAs(QWidget *parentWindow, bool notify)
     }
 
     // construct the file name
-    const QString filenameExtension = SpectacleConfig::instance()->saveImageFormat();
-    const QString mimetype = QMimeDatabase().mimeTypeForFile(QLatin1String("~/fakefile.") + filenameExtension, QMimeDatabase::MatchExtension).name();
+    const QString filenameExtension = Settings::self()->defaultSaveImageFormat().toLower();
+    const QString mimetype = QMimeDatabase().mimeTypeForFile(QStringLiteral("~/fakefile.") + filenameExtension, QMimeDatabase::MatchExtension).name();
     QFileDialog dialog(parentWindow);
     dialog.setAcceptMode(QFileDialog::AcceptSave);
     dialog.setFileMode(QFileDialog::AnyFile);
-    dialog.setDirectoryUrl(config->lastSaveAsLocation());
+    dialog.setDirectoryUrl(Settings::self()->lastSaveAsLocation().adjusted(QUrl::RemoveFilename));
     dialog.selectFile(makeAutosaveFilename() + QStringLiteral(".") + filenameExtension);
     dialog.setDefaultSuffix(QStringLiteral(".") + filenameExtension);
     dialog.setMimeTypeFilters(supportedFilters);
@@ -492,8 +516,7 @@ bool ExportManager::doSaveAs(QWidget *parentWindow, bool notify)
         if (saveUrl.isValid()) {
             if (save(saveUrl)) {
                 emit imageSaved(saveUrl);
-                config->setLastSaveAsFile(saveUrl);
-
+                Settings::setLastSaveAsLocation(saveUrl);
                 if (notify) {
                     emit forceNotify(saveUrl);
                 }
@@ -515,7 +538,7 @@ void ExportManager::doSaveAndCopy(const QUrl &url)
     if (save(savePath)) {
         QDir dir(savePath.path());
         dir.cdUp();
-        SpectacleConfig::instance()->setLastSaveFile(savePath);
+        Settings::setLastSaveLocation(savePath);
 
         doCopyToClipboard(false);
         emit imageSavedAndCopied(savePath);
@@ -523,7 +546,6 @@ void ExportManager::doSaveAndCopy(const QUrl &url)
 }
 
 // misc helpers
-
 void ExportManager::doCopyToClipboard(bool notify)
 {
     auto data = new QMimeData();
