@@ -22,8 +22,11 @@
 
 #include "KSMainWindow.h"
 
-#include "Config.h"
+#include "settings.h"
 #include "SettingsDialog/SettingsDialog.h"
+#include "SettingsDialog/GeneralOptionsPage.h"
+#include "SettingsDialog/SaveOptionsPage.h"
+#include "SettingsDialog/ShortcutsOptionsPage.h"
 
 #include <QApplication>
 #include <QClipboard>
@@ -47,7 +50,10 @@
 #include <KGuiItem>
 #include <KHelpMenu>
 #include <KIO/OpenFileManagerWindowJob>
+#include <KRun>
 #include <KLocalizedString>
+#include <KConfigGroup>
+#include <KSharedConfig>
 #include <KStandardAction>
 #include <KWindowSystem>
 
@@ -55,7 +61,7 @@ static const int DEFAULT_WINDOW_HEIGHT = 420;
 static const int DEFAULT_WINDOW_WIDTH = 840;
 static const int MAXIMUM_WINDOW_WIDTH = 1000;
 
-KSMainWindow::KSMainWindow(const Platform::GrabModes &theGrabModes, const Platform::ShutterModes &theShutterModes, QWidget *parent) :
+KSMainWindow::KSMainWindow(Platform::GrabModes theGrabModes, Platform::ShutterModes theShutterModes, QWidget *parent) :
     QDialog(parent),
     mKSWidget(new KSWidget(theGrabModes, this)),
     mDivider(new QFrame(this)),
@@ -73,6 +79,10 @@ KSMainWindow::KSMainWindow(const Platform::GrabModes &theGrabModes, const Platfo
     mScreenRecorderToolsMenu(new QMenu(this)),
     mExportMenu(new ExportMenu(this)),
     mShutterModes(theShutterModes)
+#ifdef KIMAGEANNOTATOR_FOUND
+    ,mAnnotateButton(new QToolButton(this))
+    ,mAnnotatorActive(false)
+#endif
 {
     // before we do anything, we need to set a window property
     // that skips the close/hide window animation on kwin. this
@@ -145,6 +155,31 @@ void KSMainWindow::init()
     mConfigureButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
     mDialogButtonBox->addButton(mConfigureButton, QDialogButtonBox::ResetRole);
 
+#ifdef KIMAGEANNOTATOR_FOUND
+    mAnnotateButton->setText(i18n("Annotate"));
+    mAnnotateButton->setToolTip(i18n("Add annotation to the screenshot"));
+    mAnnotateButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
+    mAnnotateButton->setIcon(QIcon::fromTheme(QStringLiteral("document-edit")));
+    connect(mAnnotateButton, &QToolButton::clicked, this, [this] {
+
+        if (mAnnotatorActive) {
+            mKSWidget->hideAnnotator();
+            mAnnotateButton->setText(i18n("Annotate"));
+        } else {
+            mKSWidget->showAnnotator();
+            mAnnotateButton->setText(i18n("Annotation done"));
+        }
+        mAnnotatorActive = !mAnnotatorActive;
+
+        mToolsButton->setEnabled(!mAnnotatorActive);
+        mSendToButton->setEnabled(!mAnnotatorActive);
+        mClipboardButton->setEnabled(!mAnnotatorActive);
+        mSaveButton->setEnabled(!mAnnotatorActive);
+    });
+
+    mDialogButtonBox->addButton(mAnnotateButton, QDialogButtonBox::ActionRole);
+#endif
+
     KGuiItem::assign(mToolsButton, KGuiItem(i18n("Tools")));
     mToolsButton->setIcon(QIcon::fromTheme(QStringLiteral("tools"),
                                            QIcon::fromTheme(QStringLiteral("application-menu"))));
@@ -174,7 +209,7 @@ void KSMainWindow::init()
 
     // the tools menu
     mToolsMenu->addAction(QIcon::fromTheme(QStringLiteral("document-open-folder")),
-                          i18n("Open Screenshots Folder"),
+                          i18n("Open Default Screenshots Folder"),
                           this, &KSMainWindow::openScreenshotsFolder);
     mToolsMenu->addAction(KStandardAction::print(this, &KSMainWindow::showPrintDialog, this));
     mScreenRecorderToolsMenu = mToolsMenu->addMenu(i18n("Record Screen"));
@@ -228,11 +263,11 @@ void KSMainWindow::init()
 
     // message: open containing folder
     mOpenContaining = new QAction(QIcon::fromTheme(QStringLiteral("document-open-folder")), i18n("Open Containing Folder"), mMessageWidget);
-    connect(mOpenContaining, &QAction::triggered, [=] { KIO::highlightInFileManager({SpectacleConfig::instance()->lastSaveFile()});});
+    connect(mOpenContaining, &QAction::triggered, [=] { KIO::highlightInFileManager({Settings::lastSaveLocation()});});
 
     mHideMessageWidgetTimer = new QTimer(this);
-    connect(mHideMessageWidgetTimer, &QTimer::timeout,
-            mMessageWidget, &KMessageWidget::animatedHide);
+//     connect(mHideMessageWidgetTimer, &QTimer::timeout,
+//             mMessageWidget, &KMessageWidget::animatedHide);
     mHideMessageWidgetTimer->setInterval(10000);
     // done with the init
 }
@@ -249,19 +284,17 @@ int KSMainWindow::windowWidth(const QPixmap &pixmap) const
     int alignedWindowWidth = qMin(mKSWidget->imagePaddingWidth() + imageWidth, MAXIMUM_WINDOW_WIDTH);
     alignedWindowWidth += layout()->contentsMargins().left() + layout()->contentsMargins().right();
     alignedWindowWidth += 2; // margins is removing 1 - 1 pixel for some reason
-
     return alignedWindowWidth;
 }
 
 void KSMainWindow::setDefaultSaveAction()
 {
-    switch (SpectacleConfig::instance()->lastUsedSaveMode()) {
-    case SaveMode::SaveAs:
-    default:
+    switch (Settings::lastUsedSaveMode()) {
+    case Settings::SaveAs:
         mSaveButton->setDefaultAction(mSaveAsAction);
         mSaveButton->setText(i18n("Save As..."));
         break;
-    case SaveMode::Save:
+    case Settings::Save:
         mSaveButton->setDefaultAction(mSaveAction);
         break;
     }
@@ -367,31 +400,7 @@ void KSMainWindow::showPrintDialog()
 
 void KSMainWindow::openScreenshotsFolder()
 {
-    // Highlight last screenshot in file manager if user saved at least once ever
-    // (since last save and saveas file names are stored in spectaclerc), otherwise,
-    //   if in save mode, open default save location from configure > save > location
-    //   if in save as mode, open last save as files location
-    // failsafe for either option is default save location from configure > save > location
-    SpectacleConfig *cfgManager = SpectacleConfig::instance();
-    ExportManager *exportManager = ExportManager::instance();
-    QUrl location;
-
-    switch(cfgManager->lastUsedSaveMode()) {
-    case SaveMode::Save:
-        location = cfgManager->lastSaveFile();
-        if (!exportManager->isFileExists(location)) {
-            location = cfgManager->defaultSaveLocation();
-        }
-        break;
-    case SaveMode::SaveAs:
-        location = cfgManager->lastSaveAsFile();  // already has a "/" at the end
-        if (!exportManager->isFileExists(location)) {
-            location = cfgManager->lastSaveAsLocation();
-        }
-        break;
-    }
-
-    KIO::highlightInFileManager({location});
+    new KRun(Settings::defaultSaveLocation(), this);
 }
 
 void KSMainWindow::quit(const QuitBehavior quitBehavior)
@@ -464,7 +473,7 @@ void KSMainWindow::showImageSharedFeedback(bool error, const QString &message)
 
 void KSMainWindow::copy()
 {
-    const bool quitChecked = SpectacleConfig::instance()->quitAfterSaveOrCopyChecked();
+    const bool quitChecked = Settings::quitAfterSaveCopyExport();
     ExportManager::instance()->doCopyToClipboard();
     if (quitChecked) {
         quit(QuitBehavior::QuitExternally);
@@ -477,10 +486,18 @@ void KSMainWindow::imageCopied()
                       KMessageWidget::Information);
 }
 
+void KSMainWindow::screenshotFailed()
+{
+    showInlineMessage(i18n("Could not take a screenshot. Please report this bug here: <a href=\"https://bugs.kde.org/enter_bug.cgi?product=Spectacle\">create a spectacle bug</a>"),
+                      KMessageWidget::Warning);
+}
+
 void KSMainWindow::showPreferencesDialog()
 {
-    SettingsDialog prefDialog(this);
-    prefDialog.exec();
+    if (KConfigDialog::showDialog(QStringLiteral("settings"))) {
+        return;
+    }
+    (new SettingsDialog(this))->show();
 }
 
 void KSMainWindow::imageSaved(const QUrl &location)
@@ -503,10 +520,10 @@ void KSMainWindow::imageSavedAndCopied(const QUrl &location)
 
 void KSMainWindow::save()
 {
-    SpectacleConfig::instance()->setLastUsedSaveMode(SaveMode::Save);
+    Settings::setLastUsedSaveMode(Settings::Save);
     setDefaultSaveAction();
 
-    const bool quitChecked = SpectacleConfig::instance()->quitAfterSaveOrCopyChecked();
+    const bool quitChecked = Settings::quitAfterSaveCopyExport();
     ExportManager::instance()->doSave(QUrl(), /* notify */ quitChecked);
     if (quitChecked) {
         quit(QuitBehavior::QuitExternally);
@@ -515,10 +532,10 @@ void KSMainWindow::save()
 
 void KSMainWindow::saveAs()
 {
-    SpectacleConfig::instance()->setLastUsedSaveMode(SaveMode::SaveAs);
+    Settings::setLastUsedSaveMode(Settings::SaveAs);
     setDefaultSaveAction();
 
-    const bool quitChecked = SpectacleConfig::instance()->quitAfterSaveOrCopyChecked();
+    const bool quitChecked = Settings::quitAfterSaveCopyExport();
     if (ExportManager::instance()->doSaveAs(this, /* notify */ quitChecked) && quitChecked) {
         quit(QuitBehavior::QuitExternally);
     }
@@ -529,7 +546,7 @@ void KSMainWindow::restoreWindowTitle()
     if (isWindowModified()) {
         setWindowTitle(i18nc("@title:window Unsaved Screenshot", "Unsaved[*]"));
     } else {
-        setWindowTitle(SpectacleConfig::instance()->lastSaveFile().fileName());
+        setWindowTitle(Settings::lastSaveLocation().fileName());
     }
 }
 
