@@ -26,6 +26,7 @@
 
 #include "SpectacleCommon.h"
 #include "Platforms/Platform.h"
+#include "Config.h"
 
 class QAction;
 class QGridLayout;
@@ -35,18 +36,25 @@ class QFormLayout;
 class QComboBox;
 class QCheckBox;
 class QLabel;
+class KConfigDialogManager;
+class QPushButton;
+class QStackedLayout;
 
 class KSImageWidget;
 class ProgressButton;
 class SmartSpinBox;
 
+namespace kImageAnnotator {
+    class KImageAnnotator;
+}
+
 class KSWidget : public QWidget
 {
     Q_OBJECT
 
 	public:
 
-    explicit KSWidget(const Platform::GrabModes &theGrabModes, QWidget *parent = nullptr);
+    explicit KSWidget(Platform::GrabModes theGrabModes, QWidget *parent = nullptr);
     virtual ~KSWidget() = default;
 
 
@@ -68,9 +76,14 @@ class KSWidget : public QWidget
     void setScreenshotPixmap(const QPixmap &thePixmap);
     void lockOnClickDisabled();
     void lockOnClickEnabled();
-    void setButtonState(State state);
+    void setButtonState(KSWidget::State state);
     void setProgress(double progress);
 
+#ifdef KIMAGEANNOTATOR_FOUND
+    void showAnnotator();
+    void hideAnnotator();
+#endif
+
     private Q_SLOTS:
 
     void newScreenshotClicked();
@@ -98,4 +111,10 @@ class KSWidget : public QWidget
     bool           mTransientWithParentAvailable { false };
     QAction       *mTakeNewScreenshotAction;
     QAction       *mCancelAction;
+    KConfigDialogManager *mConfigManager;
+    QStackedLayout *mStack                       { nullptr };
+    QWidget *placeHolder;
+#ifdef KIMAGEANNOTATOR_FOUND
+    kImageAnnotator::KImageAnnotator *mAnnotator { nullptr };
+#endif
 };
