@@ -28,10 +28,10 @@ class SmartSpinBox : public QDoubleSpinBox
 
     public:
 
-    explicit SmartSpinBox(QWidget *parent = 0);
-    QString textFromValue(double val) const Q_DECL_OVERRIDE;
+    explicit SmartSpinBox(QWidget *parent = nullptr);
+    QString textFromValue(double val) const override;
 
-    private slots:
+    private Q_SLOTS:
 
     void suffixChangeHandler(double val);
 };
